#!/usr/bin/env perl

use Rappture;

# open the XML file containing the run parameters
$driver = Rappture::RpLibrary->new($ARGV[0]);

# Set variable paths
$exampleDir = $ENV{'TOOLDIR'} . "/examples";
$rapDir = $ENV{'TOOLDIR'} . "/rappture";

#set filenames
$run_number = $driver->get("input.choice(Run_Number).current");
$save_workspace = $driver->get("input.boolean(Save_Workspace).current");
$configname="T2_Input.txt";
$outputname="T2_Output.txt";
$windname="T2_Wind.txt";
if ($save_workspace eq "yes"){
$configname="T2_Input".$run_number.".txt";
$outputname="T2_Output".$run_number.".txt";
$windname="T2_Wind".$run_number.".txt"; }



#Find out what language to print in
$run_number = $driver->get("input.string(Run_Number).current");
$language = $driver->get("input.string(language).current");

# Obtain run parameters
$Plume_Height = $driver->get("input.(Plume_Height).current");
$Eruption_Mass = $driver->get("input.(Eruption_Mass).current");
$Max_Grainsize = $driver->get("input.(Max_Grainsize).current");
$Min_Grainsize = $driver->get("input.(Min_Grainsize).current");
$Median_Grainsize = $driver->get("input.(Median_Grainsize).current");
$STD_Grainsize = $driver->get("input.(STD_Grainsize).current");
$Vent_Easting = $driver->get("input.(Vent_Easting).current");
$Vent_Northing = $driver->get("input.(Vent_Northing).current");
$Vent_Elevation = $driver->get("input.(Vent_Elevation).current");
$Eddy_Const = $driver->get("input.(Eddy_Constant).current");
$Diffusion_Coefficient = $driver->get("input.(Diffusion_Coefficient).current");
$Fall_Time_Threshold = $driver->get("input.(Fall_Time_Threshold).current");
$Lithic_Density = $driver->get("input.(Lithic_Density).current");
$Pumice_Density = $driver->get("input.(Pumice_Density).current");
$Col_Steps = $driver->get("input.(Column_Steps).current");
$Plume_Model = $driver->get("input.(Plume_Model).current");
$Plume_Ratio = $driver->get("input.(Plume_Ratio).current");
$Save_File = $driver->get("input.(Save_File).current");
$Filename = $driver->get("input.(Filename).current");

$wind = $driver->get("input.(wind).current");
$grid = $driver->get("input.(grid).current");
$config = $driver->get("input.(config).current");
$configexist = length ($config);

# Set the wind field
if ($wind eq "constant") {
    $wind = "";
    open(IN, "$exampleDir/w_constant.wind");
    while (<IN>) { $wind .= $_; }
    close(IN);}

if ($wind eq "stratified") {
    $wind = "";
    open(IN, "$exampleDir/w_stratified.wind");
    while (<IN>) { $wind .= $_; }
    close(IN);}

if ($wind eq "Random_CN"){
$range = 366;
$wind_directory = "$exampleDir/wind_CN1992";
$random_number = int(rand($range));
$windfilename="$wind_directory/wind$random_number";
local( $/, *EXWIND ) ;
open EXWIND, "$windfilename";
$wind=<EXWIND>;
close(EXWIND);}

if ($wind eq "Random_I"){
$range = 366;
$wind_directory = "$exampleDir/wind_I1968";
$random_number = int(rand($range));
$windfilename="$wind_directory/wind$random_number";
local( $/, *EXWIND ) ;
open EXWIND, "$windfilename";
$wind=<EXWIND>;
close(EXWIND);}

if ($wind eq "Random_P"){
$range = 366;
$wind_directory = "$exampleDir/wind_P1968";
$random_number = int(rand($range));
$windfilename="$wind_directory/wind$random_number";
local( $/, *EXWIND ) ;
open EXWIND, "$windfilename";
$wind=<EXWIND>;
close(EXWIND);}



# If the user selected the example files, load the data

# Cerro Negro
if ($grid eq "Cerro_Negro") {
    $grid = "";
    open(IN, "$exampleDir/gridCN100.pos");
    while (<IN>) { $grid .= $_; }
    close(IN);}

# Irazu
if ($grid eq "Irazu") {
    $grid = "";
    open(IN, "$exampleDir/gridIrazu100.pos");
    while (<IN>) { $grid .= $_; }
    close(IN);}

# Pululagua
if ($grid eq "Pululagua") {
    $grid = "";
    open(IN, "$exampleDir/gridPu100.pos");
    while (<IN>) { $grid .= $_; }
    close(IN);}

# Save the data to the respective files so tephra can access them
open(WIND, ">$windname");
print WIND "$wind\n";
close(WIND);

open(GRID, ">grid.txt");
print GRID "$grid\n";
close(GRID);

# Create configuration file
$configtoprint="PLUME_HEIGHT $Plume_Height\n
ERUPTION_MASS $Eruption_Mass\n
MAX_GRAINSIZE $Max_Grainsize\n
MIN_GRAINSIZE $Min_Grainsize\n
MEDIAN_GRAINSIZE $Median_Grainsize\n
STD_GRAINSIZE $STD_Grainsize\n
VENT_EASTING $Vent_Easting\n
VENT_NORTHING $Vent_Northing\n
VENT_ELEVATION $Vent_Elevation\n
EDDY_CONST $Eddy_Const\n
DIFFUSION_COEFFICIENT $Diffusion_Coefficient\n
FALL_TIME_THRESHOLD $Fall_Time_Threshold\n
LITHIC_DENSITY $Lithic_Density\n
PUMICE_DENSITY $Pumice_Density\n
COL_STEPS $Col_Steps\n
PLUME_MODEL $Plume_Model\n
PLUME_RATIO $Plume_Ratio";

if ($configexist > 100){
open CONFIG, ">$configname";
print CONFIG "$config\n";
close(CONFIG);}
else {
open FILE1, ">$configname";
print FILE1 "$configtoprint";
close(FILE1);}

#Run tephra2: output script: Tephra2out#.dat
if ($language eq "English"){print"\nTEPHRA2 is calculating the mass accumulation of tephra at the supplied points about the volcano...\n\n";}
if ($language eq "Spanish"){print"\nTEPHRA2 esta calculando la masa de tefra acumulada en los puntos especificados...\n\n";}
system"tephra2 $configname grid.txt $windname";


###################################  Creating isopach map #############################
if ($language eq "English"){print"\nFormatting Tephra2 output for plotting\n";}
if ($language eq "Spanish"){print"\nFormateando los resultados de Tephra2 para graficarlos\n";}

system "perl $rapDir/parse_tephra.out.pl Tephra2out.dat > tephra2.out.utm";
system "mv Tephra2out.dat $outputname";
system "cp $rapDir/.gmt* .";

$out = "location_map.eps";
$tephra = "tephra2.out.utm";

# Determine the max and min dimensions of the input grid and
# The max and min dimensions of the map where >0.1kg/m^2 of tephra accumulated
open(INP,"tephra2.out.utm");
$ii=-1;
$i=0;
while (<INP>) {
 ($east, $north, $mass) = split " ", $_;
if($ii>(-1)){
	if ($mass >= 0.1 )
		{ $xx[$ii]=$east;
		  $yy[$ii]=$north;
		  $masa[$ii]=$mass;

  		  $xplot[$i]=$east;
		  $yplot[$i]=$north;
		  $i++;}
	else
		{ $xx[$ii]=$east;
		  $yy[$ii]=$north;
		  $masa[$ii]=$mass;}}
	  $ii++;
}
close(INP);

@xsortm=sort(@xplot);
@ysortm=sort(@yplot);
@xsort=sort(@xx);
@ysort=sort(@yy);
$xminm = $xsortm[0];
$xmaxm = $xsortm[-1];
$yminm = $ysortm[0];
$ymaxm = $ysortm[-1];
$xmin = $xsort[0];
$xmax = $xsort[-1];
$ymin = $ysort[0];
$ymax = $ysort[-1];
$delta_x=($xmax-$xmin)/500;

# Use GMT to make the plot
if ($language eq "English"){print"\nCreating the location map and contouring the data. . . \n";}
system "psbasemap --ANNOT_FONT_PRIMARY=2 --LABEL_FONT_SIZE=10 --ANNOT_FONT_SIZE=8 --D_FORMAT=%.0f -JX6i -R$xminm/$xmaxm/$yminm/$ymaxm -Ba10000:'':/a10000:'':/:.'Isopach Map': -P -V -K > $out";
system "surface --D_FORMAT=%g $tephra -Gtephra.grd -I$delta_x -R$xmin/$xmax/$ymin/$ymax -V";
system "grdcontour tephra.grd -C$rapDir/contours -G0.5i+r0.1i -A100+f7+s4+kblack -Wa0.5p,black -Wc.18p,black -L0.1/10e6 -JX6i -O -R$xminm/$xmaxm/$yminm/$ymaxm >> $out";


###################################### Done Creating Isopach Map  ####################

#Remove unnecesary files
system "ps2raster $out -A -Tg";
system "rm *.eps";
system "rm grid.txt";
system "rm node_";
system "rm tephra.grd";
system "rm .gmt*";
system "rm tephra2.out.utm";


#Load Output into graphical user interface (rappture)
local( $/, *OUTPUT ) ;
open OUTPUT, "$outputname";
$test=<OUTPUT>;

$driver->put("output.image(outa).about.label","Isopach Map: $Volcano",0);
$driver->putFile("output.image(outa).current", "location_map.png", 1, 0);

$driver->put("output.string(outb).about.label","TEPHRA2 Output",0);
$driver->put("output.string(outb).current",$test,0);

$driver->put("output.string(outc).about.label","TEPHRA2 Model Inputs",0);
$driver->put("output.string(outc).current",$configtoprint,0);

$driver->put("output.string(outd).about.label","TEPHRA2 Wind Input",0);
$driver->put("output.string(outd).current",$wind,0);

$driver->result();

system "rm location_map.png";
if ($save_workspace eq "no"){system "rm T2_Output.txt"; system "rm T2_Wind.txt"; system "rm T2_Input.txt"; }


exit;
